const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');

module.exports = {
    entry: './index.js', // 单入口文件
    output: {
        filename: 'bundle.js', // 输出的文件名
        path: path.resolve(__dirname, 'dist'), // 输出目录
        clean: true, // 构建前清空 dist 目录
        library: "MyLibrary",  // 将导出的内容绑定到全局对象
        libraryTarget: "window", // 确保挂载到全局对象 window
    },
    mode: 'development', // 开发模式
    resolve: {
        fallback: {
            "buffer": require.resolve("buffer/"),
            "stream": require.resolve("stream-browserify"),
            "string_decoder": require.resolve("string_decoder/"),
            "timers": require.resolve("timers-browserify"),
        }
    },
    devServer: {
        static: './dist', // 告诉 webpack-dev-server 使用 dist 目录
        open: true, // 自动打开浏览器
        hot: true, // 开启热更新
    },
    plugins: [
        new HtmlWebpackPlugin({
            template: './index.html', // 指定 HTML 模板，其实可以这么理解，就是我们使用webpack需要引入js、css资源，通过这个插件，可以在dist下面生成同样的html，它自动把引入js、css的动作做了，直接运行dist下面的html就可以了
            filename: 'index.html',//更换的名称
        }),
    ],
    module: {
        rules: [
            {
                test: /\.js$/,
                exclude: /node_modules/,
                use: {
                    loader: "babel-loader",
                },
            },
        ],
    },
};
